import asyncio
import logging
import sys
import time
import traceback
from threading import Thread

logging.basicConfig(level=logging.DEBUG,
                    format="%(asctime)s %(threadName)-10s - %(name)-8s %(levelname).1s - %(message)s")
loop: asyncio.BaseEventLoop = None


def run_loop():
    global loop
    loop = asyncio.new_event_loop()
    loop.set_debug(True)
    try:
        logging.info("running loop")
        loop.run_forever()
    finally:
        logging.info("loop ended")


def schedule_later(delay, msg):
    logging.info("schedule_later: %s", msg)
    loop.call_at(loop.time() + delay, logging.info, "schedule_later:called_at %s", msg)


logging.info("Starting loop thread")
loop_thread = Thread(target=run_loop, name="run_loop")
loop_thread.start()
logging.info("Started loop thread")
time.sleep(1)

logging.info("Scheduling something in the far future")
loop.call_soon_threadsafe(schedule_later, 30, "far future")
time.sleep(1)

logging.info("Scheduling something in the near future")
loop.call_soon_threadsafe(schedule_later, 10, "near future")
time.sleep(1)

logging.info("Stopping loop")
loop.call_soon_threadsafe(loop.stop)
# loop.stop()

time.sleep(1)
logging.info("Current task %s in loop %s in loop thread %s", asyncio.Task.current_task(loop=loop), loop, loop_thread)
logging.info("%s tasks scheduled for later execution: %s", len(loop._scheduled), loop._scheduled)
if loop_thread.is_alive():
    logging.info("Loop thread stack: %s", "".join(traceback.format_stack(sys._current_frames()[loop_thread.ident])))

logging.info("Waiting for loop join")
loop_thread.join()
logging.info("loop joined")
